//
//  Authorization.m
//  Authorization
//
//  Created by ed on Wed Nov 14 2001.
//  Copyright (c) 2001 Septicus Software. All rights reserved.
//  $Author: ed $
//  $Id: Authorization.m,v 1.5 2002/10/28 21:18:00 ed Exp $
//  $Date: 2002/10/28 21:18:00 $
//  $Name:  $
//  $Revision: 1.5 $
//  $Log: Authorization.m,v $
//  Revision 1.5  2002/10/28 21:18:00  ed
//  fixed a bug where an exception was thrown if there was a non ASCII char in the path to the executable to be run...
//
//  Revision 1.4  2002/04/06 22:55:37  ed
//  added copyright for Septicus Software...
//
//  Revision 1.3  2002/03/23 17:24:48  ed
//  changed getToolPath to toolPath, changed includes to use Cocoa.h, not just Foundation.h
//
//  Revision 1.2  2002/02/08 18:19:24  ed
//  fixed a typo...
//
//  Revision 1.1.1.1  2002/02/08 18:15:40  ed
//  putting Authorization code into CVS
//
//  Description: Provides a Cocoa Object wrapper around Security Framework...
//

#import <Authorization/Authorization.h>

@implementation Authorization

// init the auth ref...
- (id)init
{
    if (self = [super init]){
        return self;
    }
    return nil;
}

// init the auth ref...
- (id)initWithToolPath:(NSString *)tool
{
    if (self = [super init]){
        toolPath = [[NSString alloc] initWithString:tool];
        return self;
    }
    return nil;
}

-(void)dealloc
{
    [toolPath release];
    [super dealloc];
}

-(void)setToolPath:(NSString *)tool
{
    if (toolPath){
        [toolPath release];
    }
    toolPath = [[NSString alloc] initWithString:tool];
}

-(NSString *)toolPath
{
    return toolPath;
}

-(int)runAuthenticated:(char **)arg
{
    int err = -1;
    
    if (toolPath){
        err = AuthorizationExecuteWithPrivileges(authRef,
                                                 [toolPath fileSystemRepresentation],
                                                 0, arg, NULL);
    }
    
    return err;
}

-(int)runToolAuthenticated:(NSString *)tool args:(char **)arg
{
    int err = 0;
    
    err = AuthorizationExecuteWithPrivileges(authRef,
                                            [tool cString],
                                            0, arg, NULL);
    return err;
}

- (void)clearAuthRef
{
    [self deauthenticate];

    return;
}

// is the user authenticated...?
- (BOOL)authenticate
{
    if(![self isAuthenticated])
    {
        [self fetchPassword];
    }
    return [self isAuthenticated];
}

// deauthenticate the user...
- (void)deauthenticate
{
    if(authRef)
    {
        AuthorizationFree(authRef,
                          kAuthorizationFlagDestroyRights);
        authRef = NULL;
    }
}

// yes, no...?
- (BOOL)isAuthenticated
{
    AuthorizationRights rights;
    AuthorizationFlags flags;
    AuthorizationItem items[1];
    OSStatus err = 0;
    BOOL authorized = NO;

    items[0].name = kAuthorizationRightExecute;
    if (toolPath){
        items[0].value = (char *)[toolPath fileSystemRepresentation];
        items[0].valueLength = strlen([toolPath fileSystemRepresentation]);
    }
    items[0].flags = 0;

    rights.count=1;
    rights.items = items;

    if(authRef == NULL)
    {
	flags = kAuthorizationFlagInteractionAllowed 
	    | kAuthorizationFlagExtendRights;
    
        err = AuthorizationCreate(NULL,
                                kAuthorizationEmptyEnvironment, flags,
			  &authRef);
    }
    
    if (err == errAuthorizationSuccess)
    {
        flags = kAuthorizationFlagExtendRights;

        err = AuthorizationCopyRights(authRef, &rights,
                                      kAuthorizationEmptyEnvironment,
                                      flags, NULL);

        authorized = (errAuthorizationSuccess == err);
    }
    
    return authorized;
}

// if not, get passwd...
- (BOOL)fetchPassword
{
    AuthorizationRights rights;
    AuthorizationFlags flags;
    AuthorizationItem items[1];
    OSStatus err = 0;
    BOOL authorized = NO;

    items[0].name = kAuthorizationRightExecute;
    if (toolPath){
        items[0].value = (char *)[toolPath fileSystemRepresentation];
        items[0].valueLength = strlen([toolPath fileSystemRepresentation]);
    }
    items[0].flags = 0;

    rights.count=1;
    rights.items = items;
    
    flags = kAuthorizationFlagInteractionAllowed 
	| kAuthorizationFlagExtendRights;

    err = AuthorizationCopyRights(authRef, &rights,
                                  kAuthorizationEmptyEnvironment,
                                  flags, NULL);
    authorized = (errAuthorizationSuccess == err);

    return authorized;
}

@end
